module MdocsKramdown
  module Parser
    module FencedAdmonitions
      FENCED_ADMONITIONS_START = /^\s{0,}!{3}/x.freeze

      FENCED_ADMONITIONS_MATCH = %r{
        ^\s{0,}*(?<delimeter>!{3})  # line must start with [0..] spacing and !!!
          \s*                       # any amount of trailling whitespace
          (?<type>[^\s]+)?          # type of admoniation any except space. Ex: info,class1
          \s*                       # any amount of trailling whitespace
          ("(?<title>[^\n]+)")?     # title of the admoniation
          \s*                       # any amount of trailling whitespace
        \n                          # heading line completed
        (?<content>(\s{4}.*\n)+)    # content must starts with 4 spaces
        \n
      }x.freeze

      def self.included(klass)
        klass.define_parser(:fenced_admonitions, FENCED_ADMONITIONS_START)
      end

      def parse_fenced_admonitions
        if @src.check(FENCED_ADMONITIONS_MATCH)
          start_line_number = @src.current_line_number
          @src.pos += @src.matched_size

          el = new_block_el(:blockquote, nil, { types: @src[:type].to_s.split(','), title: @src[:title] }, location: start_line_number, admonition: true)

          content = parse_inner_alert_content(@src[:content].gsub(/^\s{4}/, ''))
          el.children = content
          @tree.children << el
          true
        else
          false
        end
      end

      def parse_inner_alert_content(content)
        parsed = ::Kramdown::Parser::MdocsKramdown.parse(content, @options)
        parsed[0].children
      end
    end
  end
end
